@@ -9,7 +9,7 @@ from json import loads as json_loads
 from pickle import dumps as pickle_dumps
 from pickle import loads as pickle_loads
 from decimal import Decimal
-from .data_structures import ungive
+from .toolkit.data_structures import undefined
 
 
 nowdate = lambda : str(datetimeSon.now())[:10]  # -> '2021-10-24'
@@ -32,36 +32,57 @@ def GetGroupNumber(size, i):
     '''
     return ceil(i / size)
 
+_WindowsPath = type(libpath(''))
+
 def set_path(path):
-    libpath(path).mkdir(parents=True, exist_ok=True)
+    if type(path) is not _WindowsPath:
+        path = libpath(path)
+    return path.mkdir(parents=True, exist_ok=True)
+
+def set_parent_path(path):
+    if type(path) is not _WindowsPath:
+        path = libpath(path)
+    return path.parent.mkdir(parents=True, exist_ok=True)
 
 def json_chinese(data):
     return json_dumps(data, ensure_ascii=False)
 
 def write_json(fpath, data, ensure_ascii=False):
+    set_parent_path(fpath)
     data = json_dumps(data, ensure_ascii=ensure_ascii)
-    libpath(fpath).write_text(data, encoding='utf8')
+    return libpath(fpath).write_text(data, encoding='utf8')
 
-def read_json(fpath, default=ungive):
+def read_json(fpath, default=undefined):
     try:
         return json_loads(libpath(fpath).read_text(encoding='utf8'))
     except Exception as e:
-        if default is ungive:
+        if default is undefined:
             raise e
-        else:
-            return default
+        return default
 
 def write_pickle(fpath, data):
-    libpath(fpath).write_bytes(pickle_dumps(data))
+    set_parent_path(fpath)
+    return libpath(fpath).write_bytes(pickle_dumps(data))
 
-def read_pickle(fpath, default=ungive):
+def read_pickle(fpath, default=undefined):
     try:
         return pickle_loads(libpath(fpath).read_bytes())
     except Exception as e:
-        if default is ungive:
+        if default is undefined:
             raise e
-        else:
-            return default
+        return default
+
+def write_text(fpath, text):
+    set_parent_path(fpath)
+    return libpath(fpath).write_text(text, 'utf8')
+
+def read_text(fpath, default=undefined):
+    try:
+        return libpath(fpath).read_text('utf8')
+    except Exception as e:
+        if default is undefined:
+            raise e
+        return default
 
 def limit_input(prompt='', limit=None):
     if type(limit) in (list, tuple, set, dict, str):
@@ -84,7 +105,9 @@ def limit_input(prompt='', limit=None):
 # Python的三元表达式
 # ternary(O, T, F) 比 T if O else F 可读性更好不是吗?
 def ternary(obj, tv, fv):
-    return tv if obj else fv
+    if obj:
+        return tv
+    return fv
 
 # 把Python对象转化成可被JSON序列化的对象
 # 例如：从MySQL数据库提取出来的数据，可能含有Decimal、datetimeSon等数据类型，而这些类型的数据是无法JSON化的。
