@@ -1,7 +1,5 @@
 '''
 MongoDB ORM
-作者: jutooy
-邮箱: jutooy@qq.com
 基于 pymongo 开发
 '''
 
@@ -9,8 +7,8 @@ from math import ceil, floor
 from collections import deque
 from copy import deepcopy
 from decimal import Decimal
-from .data_structures import NeedNotExecute, SetAttrError, ParamError
-from .data_structures import ungive, int0, pinf, uniset, empset
+from .toolkit.data_structures import NeedNotExecute, SetAttrError, ParamError
+from .toolkit.data_structures import undefined, int0, pinf, uniset, empset
 
 
 class ColumnsError(BaseException): ...
@@ -25,8 +23,8 @@ class _Factory:
             where = where or uniset
         object.__setattr__(self, 'where', where)
 
-    def __setattr__(self, key, value): raise SetAttrError('_Factory 是不可变对象')
-    def __bool__(self): return True
+    def __setattr__(self, key, value): raise SetAttrError('_Factory是不可变对象')
+    def __bool__(self): return True  # 下面有一个 where or _Factory(uniset)
 
     def __deepcopy(self, obj):
         if obj in (uniset, empset):
@@ -38,7 +36,7 @@ class _Factory:
         b = self.__deepcopy(obj.where)
         if a is uniset: return _Factory(b)
         if b is uniset: return _Factory(a)
-        if a and b:
+        if a and b:  # a和b有可能是empset
             if set(a) & set(b):
                 return _Factory({'$and': [a, b]})
             return _Factory({**a, **b})
@@ -67,38 +65,35 @@ class _Factory:
         if where is empset: return {'$and': [{'a':1}, {'a':2}]}
         return where
 
-
-def _re(s, i=False):
-    if i:
-        return {'$regex': s, '$options': 'i'}
-    return {'$regex': s}
-
 class _list_base():
     def __init__(self, *lis, **dic):
         self.lis = lis
         self.dic = dic
 
-class _contain_all(_list_base): ...
-class _contain_any(_list_base): ...
-class _contain_zero(_list_base): ...
-class _isin(_list_base): ...
-class _notin(_list_base): ...
+class contain_all(_list_base): ...
+class contain_any(_list_base): ...
+class contain_zero(_list_base): ...
+class isin(_list_base): ...
+class notin(_list_base): ...
 
 class mo:  # 表示: mongodb_oper
-    re = _re
-    contain_all = _contain_all
-    contain_any = _contain_any
-    contain_zero = _contain_zero
-    isin = _isin
-    notin = _notin
-
+    # 所有操作都允许脱离mo独立调用
+    def re(s, i=False):
+        if i:
+            return {'$regex': s, '$options': 'i'}
+        return {'$regex': s}
+    contain_all = contain_all
+    contain_any = contain_any
+    contain_zero = contain_zero
+    isin = isin
+    notin = notin
 
 class _Filter():
 
     def __init__(self, field):
         object.__setattr__(self, 'field', field)
     
-    def __setattr__(self, key, value): raise SetAttrError('_Filter 是不可变对象')
+    def __setattr__(self, key, value): raise SetAttrError('_Filter是不可变对象')
     def __getattr__(self, name): return _Filter(f"{self.field}.{name}")
     def __lt__(self, obj): return _Factory({self.field: {'$lt': obj}})  # <
     def __le__(self, obj): return _Factory({self.field: {'$lte': obj}})  # <=
@@ -108,30 +103,30 @@ class _Filter():
 
     def __eq__(self, obj):
         typ = type(obj)
-        if typ is _contain_all:
+        if typ is contain_all:
             lis = obj.lis
             if len(lis) == 1: return _Factory({self.field: {'$elemMatch':{'$eq':lis[0]}}})
             if len(lis) > 1: return _Factory({'$and': [{self.field: {'$elemMatch':{'$eq':x}}} for x in set(lis)]})
             return _Factory(uniset)
         
-        elif typ is _contain_any:
+        elif typ is contain_any:
             lis = obj.lis
             if len(lis) == 1: return _Factory({self.field: {'$elemMatch':{'$eq':lis[0]}}})
             if len(lis) > 1: return _Factory({'$or': [{self.field: {'$elemMatch':{'$eq':x}}} for x in set(lis)]})
             return _Factory(empset)
         
-        elif typ is _contain_zero:
+        elif typ is contain_zero:
             lis = obj.lis
             if lis: return _Factory({'$nor': [{self.field: {'$elemMatch':{'$eq':x}}} for x in set(lis)]})
             return _Factory(uniset)
         
-        elif typ is _isin:
+        elif typ is isin:
             lis = obj.lis
             if not lis: return _Factory(empset)
             if len(lis) == 1: return _Factory({self.field: lis[0]})
             return _Factory({self.field: {'$in': lis}})
         
-        elif typ is _notin:
+        elif typ is notin:
             lis = obj.lis
             if not lis: return _Factory(uniset)
             if len(lis) == 1: return _Factory({self.field: {'$ne': lis[0]}})
@@ -159,17 +154,17 @@ class _msheet():
         setv(self, 'columns', columns)  # str型 或 tuple型
         setv(self, '_sort', deepcopy(_sort or {}))  # {A:True, B:False, ...}
 
-    def __setattr__(self, key, value): raise SetAttrError('_msheet 是不可变对象')
+    def __setattr__(self, key, value): raise SetAttrError('_msheet是不可变对象')
     
     def _deepcopy(self): ...
-    def _copy(self, where=ungive, columns=ungive, _sort=ungive):
+    def _copy(self, where=undefined, columns=undefined, _sort=undefined):
         return _msheet(
             mkconn = self.mkconn,
             connpool = self.connpool,  # 避免每个过滤器都与MySQL建立新的连接, 造成性能浪费
             sheet = self.sheet,
-            where = self.where if where is ungive else where,
-            columns = self.columns if columns is ungive else columns,
-            _sort = self._sort if _sort is ungive else _sort
+            where = self.where if where is undefined else where,
+            columns = self.columns if columns is undefined else columns,
+            _sort = self._sort if _sort is undefined else _sort
         )
 
     def get_conn(self):
@@ -226,7 +221,7 @@ class _msheet():
             raise SliceError('delete 方法暂时只支持 [1] 和 [:] 两种切片')
 
     def update(self, data, **param): return _MakeSlice(self._ExeUpdate, data=data, **param)
-    def _ExeUpdate(self, key, data, add_null=False, force_data=True, **param):
+    def _ExeUpdate(self, key, data, add_null=False, force_data=False, **param):
         columns = self.columns
         typec = type(columns)
         if columns == '*':
@@ -369,3 +364,7 @@ class _MongoDBColumn():
     def __getattr__(self, field): return _Filter(field=field)
 
 mc = _MongoDBColumn()
+
+re = mo.re
+# mo的所有操作都允许脱离mo独立调用
+# 为防止"re操作"被"第三方库re"覆盖, 故本行代码放到最后
